@@ -1,4 +1,4 @@
-# $Id: Inetd.pm,v 0.01 2004/01/19 22:39:44 sts Exp $
+# $Id: Inetd.pm,v 0.02 2004/01/20 12:14:09 sts Exp $
 
 package Net::Inetd;
 
@@ -7,7 +7,7 @@ use base qw(Exporter);
 use strict;
 use warnings;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 use Tie::File;
 
@@ -15,7 +15,7 @@ our $enable;
 
 our ($CONF, 
      $ENABLED, 
-     $INETD_CONF
+     $INETD_CONF,
 );
 
 $CONF = 'CONF';
@@ -47,6 +47,9 @@ sub is_enabled {
 sub enable { local $enable = 1; &_set }
 sub disable { &_set }
 
+sub dump_enabled { local $enable = 1; &_dump }
+sub dump_disabled { &_dump }
+
 sub _class_data {
     my $conf_file = $_[0];
     
@@ -63,21 +66,18 @@ sub _tie_conf {
     my ($conf, $file) = @_;
     
     tie @$conf, 'Tie::File', $$file
-      or croak qq~Could not tie $$file: $!~;
+      or croak qq~couldn't tie $$file: $!~;
 }   
 
 sub _parse_enabled {
-    my %is_enabled;
+    my (%is_enabled, @serv);     
+    _filter_conf(\@_);
     foreach (@_) {
-        next if !/(?:stream|dgram|raw|rdm|seqpacket)/;
-        my @serv = split /\s.*?/; 
-	splice @serv,1,1 if $serv[1] !~ /\w/; 
-	
+	_split_serv(\@serv, \$_);	
 	my ($serv, $prot) = (shift @serv, splice @serv,1,1);
-        ($serv) = $serv =~ /.*:(.*)/ if $serv =~ /.*:(.*)/;
 	
 	if (!/^\#/) { $is_enabled{$serv}{$prot} = 1 }
-	else { ($serv) = $serv =~ /.(.*)/; $is_enabled{$serv}{$prot} = 0 }
+	else { $is_enabled{$serv}{$prot} = 0 }
     }
     
     return \%is_enabled;
@@ -96,11 +96,13 @@ sub _set {
 	        if ($o->{$CONF}[$i] =~ /^\#/) {
 	            $o->{$CONF}[$i] = substr $o->{$CONF}[$i], 
 		      1, length $o->{$CONF}[$i];
+		    $o->{$ENABLED}{$serv}{$prot} = 1;
 		    $ret_state = 1;
 		}
             } 
 	    elsif ($o->{$CONF}[$i] !~ /^\#/) {
 	        $o->{$CONF}[$i] = '#'.$o->{$CONF}[$i];
+		$o->{$ENABLED}{$serv}{$prot} = 0;
 		$ret_state = 1;
 	    }      
 	}    
@@ -109,6 +111,43 @@ sub _set {
     return $ret_state;
 } 
 
+sub _dump {
+    my $o = $_[0];
+    
+    my @dump;
+    my @conf = @{$o->{$CONF}}; _filter_conf(\@conf);  
+    foreach (@conf) {
+        next if (($enable && $_ =~ /^\#/) 
+	  || (!$enable && $_ !~ /^\#/));
+	push @dump, $_;    
+    }
+    
+    return \@dump;   
+}
+
+sub _filter_conf {
+    my $conf = $_[0];
+    
+    my @tmp;
+    foreach (@$conf) {
+        push @tmp, $_
+	  if /(?:stream|dgram|raw|rdm|seqpacket)/;
+    }
+    @$conf = @tmp; 
+}
+
+sub _split_serv {
+    my ($serv, $line) = @_;
+    
+    @$serv = split /\s.*?/, $$line;
+    splice @$serv,1,1 if $$serv[1] !~ /\w/;
+    
+    ($$serv[0]) = $$serv[0] =~ /.*:(.*)/ 
+      if $$serv[0] =~ /:/;
+    $$serv[0] = substr $$serv[0], 1, length $$serv[0] 
+      if $$serv[0]=~ /^\#/; 
+}
+
 1;
 __END__
 
@@ -121,21 +160,25 @@ Net::Inetd - an interface to inetd.conf.
  use Net::Inetd;
 
  $Inetd = Net::Inetd->new;                      # constructor
- 
+
  if ($Inetd->is_enabled (telnet => 'tcp')) {    # disable telnet
      $Inetd->disable (telnet => 'tcp');
  }
- 
+
  if (!$Inetd->is_enabled (ftp => 'tcp')) {      # enable ftp
      $Inetd->enable (ftp => 'tcp');
  }
-     
- print $Inetd->{CONF}[6];                       # print a line
- 
- push @{$Inetd->{CONF}}, $entry;                # add a new entry
 
- shift @{$Inetd->{CONF}};                       # DANGEROUS.
- 
+ print $Inetd->{CONF}[6];                       # output a line
+
+ push @{$Inetd->{CONF}}, $service;              # add a new line
+
+ pop @{$Inetd->{CONF}};                         # NOT recommended.
+
+ foreach (@{$Inetd->dump_enabled}) {            # output enabled services
+     print "$_\n";
+ }
+
 =head1 DESCRIPTION
 
 C<Net::Inetd> is an interface to inetd's configuration file inetd.conf;
@@ -146,8 +189,8 @@ The configuration is tied as class data using C<Tie::File>.
 
 =head2 new
 
- $Inetd = Net::Inetd->new ('/etc/inetd.conf');
- 
+ $Inetd = Net::Inetd->new ('./inetd.conf');
+
 Omitting the path to inetd.conf, will cause the default
 F</etc/inetd.conf> to be used.
 
@@ -158,7 +201,7 @@ F</etc/inetd.conf> to be used.
 Checks whether a service is enlisted as enabled.
 
  $Inetd->is_enabled ($service => $protocol);
- 
+
 Returns 1 if the service is enlisted as enabled, 0 if 
 enlisted as disabled, undef if the service does not exist. 
 
@@ -167,10 +210,10 @@ enlisted as disabled, undef if the service does not exist.
 Enables a service.
 
  $Inetd->enable ($service => $protocol);
- 
+
 Returns 1 if the service has been enabled, 
 0 if no action has been taken.
- 
+
 =head2 disable
 
 Disables a service.
@@ -180,6 +223,24 @@ Disables a service.
 Returns 1 if the service has been disabled, 
 0 if no action has been taken.
 
+=head2 dump_enabled
+
+Dumps the enabled services.
+
+ $dumpref = $Inetd->dump_enabled;
+
+Returns an arrayref that consists of inetd.conf
+lines which contain enabled services.
+
+=head2 dump_disabled
+
+Dumps the disabled services.
+
+ $dumpref = $Inetd->dump_disabled;
+
+Returns an arrayref that consists of inetd.conf
+lines which contain disabled services.
+
 =head1 CLASS DATA
 
 The inetd.conf will be tied to the object as array and may be
